#!/bin/sh

# PURPOSE
# package a combination of sfa+sface into a snow leopard dmg install disk
# 
# NOTES
# we leverage the first similar packaging initially made by Baris Metin
# 
# REQUIREMENTS
# sface-skel-snow-leopard<arch>.dmg is the skeleton that has Qt and all the other third-party software
#   it is expected to be found in the local directory
#   otherwise, or if -f is specified, it gets fetched 
#     from http://mirror.onelab.eu/third-party/ 
# the script expects the taglevels for both sfa and sface, 
#   in order to retrieve the corresponding code and to label the resulting package properly
#   alternatively, a build-dir and tags file can be used instead

# the place to search for the  skeleton
SKEL_REPO=http://mirror.onelab.eu/third-party

# default is snow-leopard
DEFAULT_ARCH=snow-leopard
ALL_ARCHS="leopard snow-leopard"

# does not affect final dmg size as it's compressed
DISK_SIZE=100M

########################################
COMMAND=$(basename $0)

# extract the tag part of a GITPATH
function git_url () { echo $1 | cut -d @ -f 1 ; }
function git_tag () { echo $1 | cut -d @ -f 2 ; }

# go in $dest and retrieve $path part of codebase based on gitpath, e.g.
# git_retrieve \
#     git://git.onelab.eu/sface.git@sface-0.1-5 
#     sface 
#     /Volumes/sface-skel-sn/sface.app/Contents/Resources/
# or, with the -f (-file) option:
# git_retrieve -f \
#     git://git.onelab.eu/sface.git@sface-0.1-5 
#     macos/appIcon.icns 
#     /Volumes/sface-skel-sn/sface.app/appIcon.icns 
function git_retrieve () {
    filemode=""
    case "$1" in -f|--file) filemode=true; shift;; esac
    gitpath=$1; shift
    path=$1; shift
    dest=$1; shift

    giturl=$(git_url $gitpath)
    gittag=$(git_tag $gitpath)
    if [ -z "$filemode" ] ; then
	[ -d $dest ] || mkdir -p $dest
	pushd $dest
	git archive --remote=${giturl} ${gittag} | tar -xf - ${path}
	popd
    else
	destdir=$(dirname $dest)
	[ -d $destdir ] || mkdir -p $destdir
	git archive --remote=${giturl} ${gittag} | tar -xOf - ${path} > ${dest}
    fi
}

function die () {
    echo "$@" "- exiting" 
    exit 1
}

function package () {
    
    sface_GITPATH=$1; shift
    sfa_GITPATH=$1; shift
    arch=$1; shift

    skel_name=sface-skel-${arch}
    skel_dmg=./${skel_name}.dmg
    skel_mnt="/Volumes/${skel_name}"

    sfa_release=sfa-$(git_tag $sfa_GITPATH | sed -e s,sfa-,,)
    sface_release=sface-$(git_tag $sface_GITPATH | sed -e s,sface-,,)
    release=$(git_tag $sfa_GITPATH)-$(git_tag $sface_GITPATH)

    img_name=${sface_release}-${sfa_release}-${arch}
    img_dmg=./${img_name}.dmg
    img_raw_dmg=./${img_name}.raw.dmg
    img_mnt="/Volumes/${img_name}"

    # check we're clear
    [ -f $img_dmg ] && die "output dmg $img_dmg already exists"
    [ -f $img_raw_dmg ] && die "please clean up tmp file $img_raw_dmg"

    if [ -n "$FORCE_DOWNLOAD" ] ; then
	[ -d "$skel_mnt" ] && hdiutil unmount "$skel_mnt"
	[ -f "$skel_dmg" ] && rm -f "$skel_dmg"
    fi

    # mount the skeleton if not yet avail.
    if [ ! -d "$skel_mnt" ] ; then
        # locate skel dmg
	if [ ! -f $skel_dmg ] ; then
	    echo "Retrieving $skel_dmg from $SKEL_REPO"
	    curl -O $SKEL_REPO/$skel_dmg
	fi
	# needless to check for existence as curl will have created on anyway
	echo "Mounting skeleton"
	if ! hdiutil mount -quiet -mountpoint $skel_mnt $skel_dmg ; then
	    die "Could not mount $skel_dmg
Please check it is published at $SKEL_REPO
Also make sure to trash the current one before retrying"
	fi
    fi
    [ -d "$skel_mnt" ] || die "Could not mount skeleton $skel_dmg"

    # if image already mounted, it's an old run
    [ -d "$img_mnt" ] && die "Image already mounted on $img_mnt"

    echo "Initializing image ..."
    hdiutil create -quiet -size $DISK_SIZE -fs HFS -volname ${img_name} $img_raw_dmg 
    hdiutil attach -readwrite -mount required $img_raw_dmg 

    # copy skeleton as-is
    tar -C "$skel_mnt" --exclude .Trashes -cf - . | tar -C "$img_mnt" -xf -
    # places relative to mount point
    app=sface.app
    # xxx tmp
    skelapp_path=$(ls -d "$skel_mnt"/*.app)
    skelapp=$(basename "$skelapp_path")
    if [ "$skelapp" != "$app" ] ; then
	mv "$img_mnt/$skelapp" "$img_mnt/$app"
    fi
    # 
    resource_path=$app/Contents/Resources
    bin_path=$app/Contents/MacOS
    icon_path=$app/Contents/Resources/
    background_path=background
    # retrieve sfa/sfa and sface/sface in the image
    git_retrieve $sfa_GITPATH sfa "$img_mnt"/$resource_path/
    git_retrieve $sface_GITPATH sface "$img_mnt"/$resource_path/
    # copy binaries from sface to the bin dir
    for bin in sface.bin sface-run; do 
	# don't use the --file mode as this loses the executable bit
	git_retrieve $sface_GITPATH $bin "$img_mnt"/$bin_path
    done
    # install background and app icons
#    git_retrieve --file $sface_GITPATH macos/graphic-install-background.png "$img_mnt"/$background_path/background.png
    git_retrieve --file $sface_GITPATH macos/graphic-install-background.png "$img_mnt"/.background.png
    git_retrieve --file $sface_GITPATH macos/graphic-sfa.icns "$img_mnt"/$icon_path/appIcon.icns
    # the volume icons
    git_retrieve --file $sface_GITPATH macos/graphic-vol-sface.icns "$img_mnt"/.VolumeIcon.icns

    # instantiate version
    sed -e "s,@VERSIONTAG@,$sfa_release,g" -e "s,@SCMURL@,${sfa_GITPATH},g" \
	"$img_mnt"/$resource_path/sfa/util/version.py.in > "$img_mnt"/$resource_path/sfa/util/version.py

    # this apparently requires xcode
    /Developer/Tools/SetFile -a V "$img_mnt"/$background_path

    # clean up just in case
    find "$img_mnt" -name '*pyc' | xargs rm -f
    find "$img_mnt" -name '*~' | xargs rm -f

    if [ -n "$INTERACTIVE" ] ; then
	echo "Please tweak icons layout in $img_mnt under Finder, and press enter when done ..."
	read _
    fi

    hdiutil detach "$img_mnt"
    rm -f $img_dmg
    echo "Compressing..."
    hdiutil convert -quiet -format UDZO -imagekey zlib-level=9 -o $img_dmg $img_raw_dmg 
    # clean up the raw image
    rm -f $img_raw_dmg
    # unmount the skeleton
    hdiutil unmount "$skel_mnt"

    echo "=================================================="
    echo "Install image ready in $img_dmg"
    echo "You may publish it by running e.g."
    echo "rsync -av $img_dmg root@build.onelab.eu:/build/sface"
    echo "=================================================="

}

########################################
# either provide a build dir and tags file
# or sface-GITPATH sfa-GITPATH 
function usage () {
    echo "$COMMAND -b <build-dir> -t <tags-file>"
    echo " or"
    echo "$COMMAND -c <sface-GITPATH> -s <sfa-GITPATH>"
    echo ""
    echo "Common options"
    echo " -i : interactive : let you align icons properly in the image before it gets wrapped"
    echo " -f : force download of the skeleton package"
    echo " -a leopard|snow-leopard"
    echo " -A : run on all known archs"
    echo " -n : dry-run"
    echo " -v : verbose"
    echo " -h : this help"
}

DEFAULT_BUILD_DIR=$HOME/git/build
DEFAULT_TAGS_FILE=onelab-k32-tags.mk

function main () {
    while getopts "b:t:c:s:ifa:Anvh" opt ; do
	case $opt in 
	    b) BUILD_DIR=$OPTARG;;
	    t) TAGS_FILE=$OPTARG;;
	    c) sface_GITPATH=$OPTARG;;
	    s) sfa_GITPATH=$OPTARG;;
	    i) INTERACTIVE=true;;
	    f) FORCE_DOWNLOAD=true;;
	    a) ARCHS="$ARCHS $OPTARG";;
	    A) ARCHS="leopard snow-leopard";;
	    n) dry_run=true;;
	    v) set -x;;
	    h) usage; exit 1 ;;
	esac
    done
    shift $(($OPTIND - 1))

    [ -z "$BUILD_DIR" ] && BUILD_DIR=$DEFAULT_BUILD_DIR
    [ -z "$TAGS_FILE" ] && TAGS_FILE=$DEFAULT_TAGS_FILE
    [ -z "$ARCHS" ] && ARCHS=$DEFAULT_ARCH
    [[ -n "$@" ]] && { usage ; exit 1; }
    if [ -z "$sface_GITPATH" ] ; then
	sface_GITPATH=$(make -C $BUILD_DIR stage1=true PLDISTROTAGS=$TAGS_FILE +sface-GITPATH 2> /dev/null)
	echo "Retrieved from $BUILD_DIR/$TAGS_FILE.."
	echo " sface_GITPATH=$sface_GITPATH"
    fi
    if [ -z "$sfa_GITPATH" ] ; then
	sfa_GITPATH=$(make -C $BUILD_DIR stage1=true PLDISTROTAGS=$TAGS_FILE +sfa-GITPATH 2> /dev/null)
	echo "Retrieved from $BUILD_DIR/$TAGS_FILE.."
	echo " sfa_GITPATH=$sfa_GITPATH"
    fi
    for ARCH in $ARCHS; do 
	if [ -z "$dry_run" ] ; then
	    package $sface_GITPATH $sfa_GITPATH $ARCH
	else
	    echo "Would run package $sface_GITPATH $sfa_GITPATH $ARCH"
	fi
    done
	
}

main "$@"
